"use strict";

function call(fn) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  var validator = (fn.eadFn || fn).validator;
  if (validator) validator.apply(undefined, args);
  return {
    type: "call",
    fn: fn,
    args: args
  };
}

function callFn(fn) {
  for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    args[_key2 - 1] = arguments[_key2];
  }

  return {
    type: "callFn",
    fn: fn,
    args: args
  };
}

function callFnBound(bindThis, fn) {
  for (var _len3 = arguments.length, args = Array(_len3 > 2 ? _len3 - 2 : 0), _key3 = 2; _key3 < _len3; _key3++) {
    args[_key3 - 2] = arguments[_key3];
  }

  return {
    type: "callFn",
    fn: fn,
    bindThis: bindThis,
    args: args
  };
}

function callCallback(fn) {
  for (var _len4 = arguments.length, args = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
    args[_key4 - 1] = arguments[_key4];
  }

  return {
    type: "callCallback",
    fn: fn,
    args: args
  };
}

function callCallbackBound(bindThis, fn) {
  for (var _len5 = arguments.length, args = Array(_len5 > 2 ? _len5 - 2 : 0), _key5 = 2; _key5 < _len5; _key5++) {
    args[_key5 - 2] = arguments[_key5];
  }

  return {
    type: "callCallback",
    fn: fn,
    bindThis: bindThis,
    args: args
  };
}

function echo(message) {
  return {
    type: "echo",
    message: message
  };
}

function noop() {
  return {
    type: "noop"
  };
}

function globalVariable(name) {
  if (!name) throw new Error("name required");
  return {
    type: "globalVariable",
    name: name
  };
}

function log() {
  for (var _len6 = arguments.length, args = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
    args[_key6] = arguments[_key6];
  }

  return {
    type: "log",
    args: args
  };
}

function logError() {
  for (var _len7 = arguments.length, args = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
    args[_key7] = arguments[_key7];
  }

  return {
    type: "logError",
    args: args
  };
}

function setImmediateCmd(cmd, resetStack) {
  return {
    type: "setImmediate",
    cmd: cmd,
    resetStack: resetStack
  };
}

function setTimeoutCmd(cmd, time, resetStack) {
  return {
    type: "setTimeout",
    cmd: cmd,
    time: time,
    resetStack: resetStack
  };
}

function clearTimeoutCmd(id) {
  return {
    type: "clearTimeout",
    id: id
  };
}

function setIntervalCmd(cmd, time, resetStack) {
  return {
    type: "setInterval",
    cmd: cmd,
    time: time,
    resetStack: resetStack
  };
}

function clearIntervalCmd(id) {
  return {
    type: "clearInterval",
    id: id
  };
}

function sleep(time) {
  return {
    type: "sleep",
    time: time
  };
}

function series(cmdList) {
  return {
    type: "series",
    cmdList: cmdList
  };
}

function parallel(cmdList) {
  return {
    type: "parallel",
    cmdList: cmdList
  };
}

function envelope(cmd) {
  return {
    type: "envelope",
    cmd: cmd
  };
}

function now() {
  return {
    type: "now"
  };
}

function either(cmd, defaultValue) {
  return {
    type: "either",
    cmd: cmd,
    defaultValue: defaultValue
  };
}

function getState(path, defaultValue) {
  return {
    type: "getState",
    path: path,
    defaultValue: defaultValue
  };
}

function setState(path, value) {
  return {
    type: "setState",
    path: path,
    value: value
  };
}

function clearState() {
  return {
    type: "clearState"
  };
}

// Friendly aliases
call.fn = callFn;
call.fnBound = callFnBound;
call.callback = callCallback;
call.callbackBound = callCallbackBound;

module.exports = {
  call: call,
  callFn: callFn,
  fn: callFn,
  callFnBound: callFnBound,
  callCallback: callCallback,
  callback: callCallback,
  callCallbackBound: callCallbackBound,
  echo: echo,
  noop: noop,
  now: now,
  globalVariable: globalVariable,
  log: log,
  logError: logError,
  setImmediate: setImmediateCmd,
  setTimeout: setTimeoutCmd,
  clearTimeout: clearTimeoutCmd,
  setInterval: setIntervalCmd,
  clearInterval: clearIntervalCmd,
  sleep: sleep,
  series: series,
  parallel: parallel,
  envelope: envelope,
  either: either,
  getState: getState,
  setState: setState,
  clearState: clearState
};
//# sourceMappingURL=cmds.js.map