"use strict";

var core = require("./core");
var coreCmds = require("./cmds");
var coreInterpreters = require("./interpreters");

var interpreters = Object.assign({}, coreInterpreters);
var context = {};

function promisify(fn) {
  if (fn.eadPromisified) return fn;
  var validator = fn.validator;
  var promised = function promised() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (validator) {
      try {
        validator.apply(undefined, args);
      } catch (e) {
        return Promise.reject(e);
      }
    }
    return core.call.apply(core, [context, interpreters, fn].concat(args));
  };
  // try/catch because this is nice for reporting, but not
  // necessary for the system to function
  // Note: there is a unit test to validate this behavior
  // so errors, although swallowed here, are picked
  // up in the unit test.
  try {
    Object.defineProperty(promised, "name", {
      value: fn.name,
      writable: false
    });
  } catch (e) {}
  promised.eadFn = fn;

  promised.callWithContext = function (c) {
    for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
      args[_key2 - 1] = arguments[_key2];
    }

    if (validator) {
      try {
        validator.apply(undefined, args);
      } catch (e) {
        return Promise.reject(e);
      }
    }
    return core.call.apply(core, [Object.assign({}, context, c), interpreters, fn].concat(args));
  };

  promised.eadPromisified = true;

  return promised;
}

function call(fn) {
  for (var _len3 = arguments.length, args = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
    args[_key3 - 1] = arguments[_key3];
  }

  return promisify(fn).apply(undefined, args);
}

function doCmd(cmd) {
  return promisify( /*#__PURE__*/regeneratorRuntime.mark(function doCmd() {
    return regeneratorRuntime.wrap(function doCmd$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return cmd;

          case 2:
            return _context.abrupt("return", _context.sent);

          case 3:
          case "end":
            return _context.stop();
        }
      }
    }, doCmd, this);
  }))();
}

function setContext(c) {
  context = c;
}

function getContext() {
  return context;
}

function addToContext(c) {
  context = Object.assign({}, context, c);
}

function setInterpreters(h) {
  interpreters = h;
}

function getInterpreters() {
  return interpreters;
}

function addInterpreters(h) {
  interpreters = Object.assign({}, interpreters, h);
}

function reset() {
  interpreters = {};
  context = {};
}

function onError(fn) {
  if (typeof fn !== "function") throw new Error("onError requires a function");
  addToContext({ onError: fn });
}

module.exports = {
  cmds: coreCmds,
  interpreters: coreInterpreters,
  promisify: promisify,
  call: call,
  doCmd: doCmd,
  setContext: setContext,
  getContext: getContext,
  addToContext: addToContext,
  setInterpreters: setInterpreters,
  getInterpreters: getInterpreters,
  addInterpreters: addInterpreters,
  reset: reset,
  onError: onError
};
//# sourceMappingURL=index.js.map