"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var _require = require("./util"),
    delay = _require.delay;

function call(_ref, c) {
  var fn = _ref.fn,
      args = _ref.args;

  return c.call.apply(c, [c.context, c.interpreters, fn.eadFn || fn].concat(_toConsumableArray(args)));
}

function callFn(_ref2, c) {
  var fn = _ref2.fn,
      bindThis = _ref2.bindThis,
      args = _ref2.args;

  if (fn.eadFn) {
    return call({ fn: fn, args: args }, c);
  } else {
    return fn.call.apply(fn, [bindThis].concat(_toConsumableArray(args)));
  }
}

function callCallback(_ref3, c) {
  var fn = _ref3.fn,
      bindThis = _ref3.bindThis,
      args = _ref3.args;

  return new Promise(function (resolve, reject) {
    try {
      fn.apply(bindThis, args.concat(function (err) {
        for (var _len = arguments.length, results = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          results[_key - 1] = arguments[_key];
        }

        if (err) return reject(err);else resolve.apply(undefined, results);
      }));
    } catch (e) {
      reject(e);
    }
  });
}

function echo(_ref4) {
  var message = _ref4.message;

  return message;
}

function noop() {}

function globalVariable(_ref5) {
  var name = _ref5.name;

  // istanbul ignore next
  var g = (typeof window === "undefined" ? "undefined" : _typeof(window)) === undefined ? global : window;
  return g[name];
}

function log(_ref6) {
  var _console;

  var args = _ref6.args;

  (_console = console).log.apply(_console, _toConsumableArray(args));
}

function logError(_ref7) {
  var _console2;

  var args = _ref7.args;

  (_console2 = console).error.apply(_console2, _toConsumableArray(args));
}

function getSubStack(context, resetStack) {
  return resetStack ? Object.assign({}, context, { stack: [] }) : context;
}

function setImmediateInterpreter(_ref8, _ref9) {
  var cmd = _ref8.cmd,
      resetStack = _ref8.resetStack;
  var call = _ref9.call,
      context = _ref9.context,
      interpreters = _ref9.interpreters;

  delay(function () {
    var subcontext = getSubStack(context, resetStack);
    call(subcontext, interpreters, /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return cmd;

            case 2:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    })).catch(function (e) {
      return e;
    });
  });
}

function setTimeoutInterpreter(_ref10, _ref11) {
  var cmd = _ref10.cmd,
      time = _ref10.time,
      resetStack = _ref10.resetStack;
  var call = _ref11.call,
      context = _ref11.context,
      interpreters = _ref11.interpreters;

  return setTimeout(function () {
    var subcontext = getSubStack(context, resetStack);
    call(subcontext, interpreters, /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return cmd;

            case 2:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    })).catch(function (e) {
      return e;
    });
  }, time);
}

function clearTimeoutInterpreter(_ref12) {
  var id = _ref12.id;

  return clearTimeout(id);
}

function setIntervalInterpreter(_ref13, _ref14) {
  var cmd = _ref13.cmd,
      time = _ref13.time,
      resetStack = _ref13.resetStack;
  var call = _ref14.call,
      context = _ref14.context,
      interpreters = _ref14.interpreters;

  return setInterval(function () {
    var subcontext = getSubStack(context, resetStack);
    call(subcontext, interpreters, /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return cmd;

            case 2:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, this);
    })).catch(function (e) {
      return e;
    });
  }, time);
}

function clearIntervalInterpreter(_ref15) {
  var id = _ref15.id;

  return clearInterval(id);
}

function sleep(_ref16) {
  var time = _ref16.time;

  return new Promise(function (resolve) {
    return setTimeout(resolve, time);
  });
}

function series(_ref17, _ref18) {
  var cmdList = _ref17.cmdList;
  var call = _ref18.call,
      context = _ref18.context,
      interpreters = _ref18.interpreters;

  if (cmdList.length === 0) return [];
  return call(context, interpreters, /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
    var results, i, result;
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            results = [];
            i = 0;

          case 2:
            if (!(i < cmdList.length)) {
              _context4.next = 10;
              break;
            }

            _context4.next = 5;
            return cmdList[i];

          case 5:
            result = _context4.sent;

            results.push(result);

          case 7:
            i++;
            _context4.next = 2;
            break;

          case 10:
            return _context4.abrupt("return", results);

          case 11:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));
}

function parallel(_ref19, _ref20) {
  var cmdList = _ref19.cmdList;
  var call = _ref20.call,
      context = _ref20.context,
      interpreters = _ref20.interpreters;

  return call(context, interpreters, /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return cmdList;

          case 2:
            return _context5.abrupt("return", _context5.sent);

          case 3:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5, this);
  }));
}

function envelope(_ref21, _ref22) {
  var cmd = _ref21.cmd;
  var call = _ref22.call,
      context = _ref22.context,
      interpreters = _ref22.interpreters;

  return call(context, interpreters, /*#__PURE__*/regeneratorRuntime.mark(function _callee6() {
    return regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return cmd;

          case 2:
            return _context6.abrupt("return", _context6.sent);

          case 3:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6, this);
  })).then(function (result) {
    return {
      success: true,
      result: result
    };
  }).catch(function (e) {
    return {
      success: false,
      result: e
    };
  });
}

function either(_ref23, _ref24) {
  var cmd = _ref23.cmd,
      defaultValue = _ref23.defaultValue;
  var call = _ref24.call,
      context = _ref24.context,
      interpreters = _ref24.interpreters;

  return call(context, interpreters, /*#__PURE__*/regeneratorRuntime.mark(function _callee7() {
    return regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            _context7.prev = 0;
            _context7.next = 3;
            return cmd;

          case 3:
            return _context7.abrupt("return", _context7.sent);

          case 6:
            _context7.prev = 6;
            _context7.t0 = _context7["catch"](0);
            return _context7.abrupt("return", defaultValue);

          case 9:
          case "end":
            return _context7.stop();
        }
      }
    }, _callee7, this, [[0, 6]]);
  }));
}

function now() {
  return Date.now();
}

var state = {};

function getState(_ref25) {
  var path = _ref25.path,
      defaultValue = _ref25.defaultValue;

  if (!path) return state;
  var value = state[path];
  if (value === undefined) return defaultValue;
  return value;
}

function setState(_ref26) {
  var path = _ref26.path,
      value = _ref26.value;

  state[path] = value;
}

function clearState() {
  state = {};
}

module.exports = {
  call: call,
  callFn: callFn,
  callCallback: callCallback,
  echo: echo,
  noop: noop,
  globalVariable: globalVariable,
  log: log,
  logError: logError,
  setImmediate: setImmediateInterpreter,
  setTimeout: setTimeoutInterpreter,
  clearTimeout: clearTimeoutInterpreter,
  setInterval: setIntervalInterpreter,
  clearInterval: clearIntervalInterpreter,
  sleep: sleep,
  series: series,
  parallel: parallel,
  envelope: envelope,
  either: either,
  now: now,
  getState: getState,
  setState: setState,
  clearState: clearState
};
//# sourceMappingURL=interpreters.js.map