"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var assert = require("assert");
var curry = require("lodash/curry");
var chunk = require("lodash/chunk");

var _require = require("./tests/test-util"),
    deepEqual = _require.deepEqual,
    isError = _require.isError;

var testRunner = function testRunner(subject, expected) {
  var index = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var previousOutput = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

  var fn = subject.eadFn || subject;
  checkForExpectedTypeMismatches(expected);

  assert(fn, "The function you are trying to test is undefined.");

  var step = expected[index];

  if (step === undefined) {
    throw new Error("Your spec does not have as many steps as your function.  Are you missing a return line?");
  }

  var _step = _slicedToArray(step, 2),
      input = _step[0],
      expectedOutput = _step[1];

  if (index === 0 && fn.validator) {
    fn.validator.apply(null, input);
  }

  if (index === 0 && !Array.isArray(input)) {
    throw new Error("The first argument of the first tuple must be an array representing the arguments of the function.");
  }
  var g = void 0;
  if (fn.next) {
    g = fn;
  } else {
    g = fn.apply(null, input);
  }

  var output = void 0;
  if (isError(input)) {
    try {
      output = g.throw(input);
    } catch (e) {
      output = { value: e, done: true };
    }
  } else {
    try {
      output = g.next(input);
    } catch (e) {
      output = { value: e, done: true };
    }
  }

  var endOnError = isError(output.value);

  try {
    deepEqual(output.value, expectedOutput);
    if (endOnError) return;
  } catch (e) {
    var stepError = "Error on Step " + (index + 1);
    e.stack = stepError + "\n\n" + e.stack;
    throw e;
  }

  if (!output.done || index + 1 < expected.length) {
    testRunner(g, expected, index + 1, output.value);
  }
};

var checkForExpectedTypeMismatches = function checkForExpectedTypeMismatches(expected) {
  if (!Array.isArray(expected)) {
    throw new Error("Your spec must return an array of tuples.  It is currently returning a value of type \"" + (typeof expected === "undefined" ? "undefined" : _typeof(expected)) + "\".");
  }
  for (var i = 0; i < expected.length; i++) {
    if (i + 1 >= expected.length) return;
    var output = expected[i][1];
    var nextInput = expected[i + 1][0];

    if (Array.isArray(output)) {
      assert(Array.isArray(nextInput), "If an array of actions is yielded, it should return an array of results.");
    }
  }
};

var testFn = function testFn(fn, spec) {
  return function () {
    var expectedLog = spec();
    testRunner(fn, expectedLog);
  };
};

var testFnV2 = function testFnV2(fn, spec) {
  return function () {
    var expectedLog = spec();
    var flat = expectedLog.reduce(function (p, step, index, log) {
      if (index === 0 || index === log.length - 1) {
        p.push(step);
        return p;
      }

      p.push(step[0]);
      p.push(step[1]);
      return p;
    }, []);
    var v1Log = chunk(flat, 2);
    testRunner(fn, v1Log);
  };
};

// Semantic test builder
var args = function args() {
  for (var _len = arguments.length, fnArgs = Array(_len), _key = 0; _key < _len; _key++) {
    fnArgs[_key] = arguments[_key];
  }

  var t = [[fnArgs]];
  return {
    yieldCmd: yieldCmd(t),
    cmd: yieldCmd(t),
    throws: throwAfterCmdReturns(t),
    returns: returns(t)
  };
};

var yieldCmd = function yieldCmd(t) {
  return function (v) {
    t[t.length - 1][1] = v;
    return {
      yieldReturns: yieldReturns(t),
      result: yieldReturns(t),
      yieldThrows: yieldReturns(t),
      interpreterThrows: yieldReturns(t),
      throws: yieldReturnThrows(t),
      returns: returnCmdResult(t)
    };
  };
};

var yieldReturns = function yieldReturns(t) {
  return function (v) {
    t[t.length] = [v];

    return {
      yieldCmd: yieldCmd(t),
      cmd: yieldCmd(t),
      throws: throwAfterCmdReturns(t),
      returns: returns(t)
    };
  };
};

var returnCmdResult = function returnCmdResult(t) {
  return function (v) {
    t[t.length] = [v, v];
    return t;
  };
};

var yieldReturnThrows = function yieldReturnThrows(t) {
  return function (v) {
    t[t.length] = [v, v];
    return t;
  };
};

var throwAfterCmdReturns = function throwAfterCmdReturns(t) {
  return function (v) {
    t[t.length - 1][1] = v;
    return t;
  };
};

var returns = function returns(t) {
  return function (a) {
    t[t.length - 1][1] = a;
    return t;
  };
};

// Modified tuples
function alt() {}

module.exports = {
  testRunner: testRunner,
  testFn: curry(testFn, 2),
  testFnV2: curry(testFnV2, 2),
  alt: alt,
  args: args
};
//# sourceMappingURL=test.js.map