"use strict";

var _require = require("../index"),
    promisify = _require.promisify;

var cmds = require("../cmds");

var _require2 = require("./test-util"),
    sleep = _require2.sleep;

test("globalVariable() should return a global variable", async function () {
  var _marked = /*#__PURE__*/regeneratorRuntime.mark(testGlobalVariable);

  var testValue = Math.random().toString();
  process.env.testValue = testValue;
  function testGlobalVariable() {
    var _ref, env;

    return regeneratorRuntime.wrap(function testGlobalVariable$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return cmds.globalVariable("process");

          case 2:
            _ref = _context.sent;
            env = _ref.env;
            return _context.abrupt("return", env.testValue);

          case 5:
          case "end":
            return _context.stop();
        }
      }
    }, _marked, this);
  }
  var actual = await promisify(testGlobalVariable)();
  expect(actual).toEqual(testValue);
});

test("call() should call an effects-as-data function", async function () {
  var _marked2 = /*#__PURE__*/regeneratorRuntime.mark(testCallee),
      _marked3 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee(message) {
    return regeneratorRuntime.wrap(function testCallee$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return cmds.echo("foo" + message);

          case 2:
            return _context2.abrupt("return", _context2.sent);

          case 3:
          case "end":
            return _context2.stop();
        }
      }
    }, _marked2, this);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return cmds.call(testCallee, "bar");

          case 2:
            return _context3.abrupt("return", _context3.sent);

          case 3:
          case "end":
            return _context3.stop();
        }
      }
    }, _marked3, this);
  }
  var actual = await promisify(testCall)();
  var expected = "foobar";
  expect(actual).toEqual(expected);
});

test("call() should handle errors", async function () {
  var _marked4 = /*#__PURE__*/regeneratorRuntime.mark(testCallee),
      _marked5 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee() {
    return regeneratorRuntime.wrap(function testCallee$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            throw new Error("oops");

          case 1:
          case "end":
            return _context4.stop();
        }
      }
    }, _marked4, this);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return cmds.call(testCallee);

          case 2:
            return _context5.abrupt("return", _context5.sent);

          case 3:
          case "end":
            return _context5.stop();
        }
      }
    }, _marked5, this);
  }
  try {
    await promisify(testCall)();
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  throw new Error("testCall did not throw");
});

test("callFn() should call a promise-returning function", async function () {
  var _marked6 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee(message) {
    return Promise.resolve("foo" + message);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return cmds.callFn(testCallee, "bar");

          case 2:
            return _context6.abrupt("return", _context6.sent);

          case 3:
          case "end":
            return _context6.stop();
        }
      }
    }, _marked6, this);
  }
  var actual = await promisify(testCall)();
  var expected = "foobar";
  expect(actual).toEqual(expected);
});

test("callFn() should call a non-promise-returning function", async function () {
  var _marked7 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee(message) {
    return "foo" + message;
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            _context7.next = 2;
            return cmds.callFn(testCallee, "bar");

          case 2:
            return _context7.abrupt("return", _context7.sent);

          case 3:
          case "end":
            return _context7.stop();
        }
      }
    }, _marked7, this);
  }
  var actual = await promisify(testCall)();
  var expected = "foobar";
  expect(actual).toEqual(expected);
});

test("fn() should call a promise-returning function", async function () {
  var _marked8 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee(message) {
    return Promise.resolve("foo" + message);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            _context8.next = 2;
            return cmds.fn(testCallee, "bar");

          case 2:
            return _context8.abrupt("return", _context8.sent);

          case 3:
          case "end":
            return _context8.stop();
        }
      }
    }, _marked8, this);
  }
  var actual = await promisify(testCall)();
  var expected = "foobar";
  expect(actual).toEqual(expected);
});

test("callFn() should handle rejected promises", async function () {
  var _marked9 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee() {
    return Promise.reject(new Error("oops"));
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            _context9.next = 2;
            return cmds.callFn(testCallee, "bar");

          case 2:
            return _context9.abrupt("return", _context9.sent);

          case 3:
          case "end":
            return _context9.stop();
        }
      }
    }, _marked9, this);
  }
  try {
    await promisify(testCall)();
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  throw new Error("testCall did not reject");
});

test("callFn() should handle errors", async function () {
  var _marked10 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee() {
    throw new Error("oops");
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            _context10.next = 2;
            return cmds.callFn(testCallee, "bar");

          case 2:
            return _context10.abrupt("return", _context10.sent);

          case 3:
          case "end":
            return _context10.stop();
        }
      }
    }, _marked10, this);
  }
  try {
    await promisify(testCall)();
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  throw new Error("testCall did not throw");
});

test("callFnBound() should call a function with `this`", async function () {
  var _marked11 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var context = {
    message: "bar"
  };
  function testCallee() {
    return Promise.resolve("foo" + this.message);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            _context11.next = 2;
            return cmds.callFnBound(context, testCallee);

          case 2:
            return _context11.abrupt("return", _context11.sent);

          case 3:
          case "end":
            return _context11.stop();
        }
      }
    }, _marked11, this);
  }
  var actual = await promisify(testCall)();
  var expected = "foobar";
  expect(actual).toEqual(expected);
});

test("callFnBound() should handle errors", async function () {
  var _marked12 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var context = {
    message: "bar"
  };
  function testCallee() {
    throw new Error("oops");
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context12) {
      while (1) {
        switch (_context12.prev = _context12.next) {
          case 0:
            _context12.next = 2;
            return cmds.callFnBound(context, testCallee);

          case 2:
            return _context12.abrupt("return", _context12.sent);

          case 3:
          case "end":
            return _context12.stop();
        }
      }
    }, _marked12, this);
  }
  try {
    await promisify(testCall)();
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  throw new Error("testCall did not throw");
});

test("callCallback() should call a callback function", async function () {
  var _marked13 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee(message, done) {
    done(null, "foo" + message);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            _context13.next = 2;
            return cmds.callCallback(testCallee, "bar");

          case 2:
            return _context13.abrupt("return", _context13.sent);

          case 3:
          case "end":
            return _context13.stop();
        }
      }
    }, _marked13, this);
  }
  var actual = await promisify(testCall)();
  var expected = "foobar";
  expect(actual).toEqual(expected);
});

test("callback() should call a callback function", async function () {
  var _marked14 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee(message, done) {
    done(null, "foo" + message);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context14) {
      while (1) {
        switch (_context14.prev = _context14.next) {
          case 0:
            _context14.next = 2;
            return cmds.callback(testCallee, "bar");

          case 2:
            return _context14.abrupt("return", _context14.sent);

          case 3:
          case "end":
            return _context14.stop();
        }
      }
    }, _marked14, this);
  }
  var actual = await promisify(testCall)();
  var expected = "foobar";
  expect(actual).toEqual(expected);
});

test("callCallback() should handle errors", async function () {
  var _marked15 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee(done) {
    done(new Error("oops"));
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context15) {
      while (1) {
        switch (_context15.prev = _context15.next) {
          case 0:
            _context15.next = 2;
            return cmds.callCallback(testCallee);

          case 2:
            return _context15.abrupt("return", _context15.sent);

          case 3:
          case "end":
            return _context15.stop();
        }
      }
    }, _marked15, this);
  }
  try {
    await promisify(testCall)();
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  throw new Error("testCall did not throw");
});

test("callCallback() should handle thrown errors", async function () {
  var _marked16 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee(done) {
    throw new Error("oops");
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context16) {
      while (1) {
        switch (_context16.prev = _context16.next) {
          case 0:
            _context16.next = 2;
            return cmds.callCallback(testCallee);

          case 2:
            return _context16.abrupt("return", _context16.sent);

          case 3:
          case "end":
            return _context16.stop();
        }
      }
    }, _marked16, this);
  }
  try {
    await promisify(testCall)();
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  throw new Error("testCall did not throw");
});

test("callCallbackBound() should call a callback function with `this`", async function () {
  var _marked17 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var context = {
    message: "bar"
  };
  function testCallee(done) {
    done(null, "foo" + this.message);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context17) {
      while (1) {
        switch (_context17.prev = _context17.next) {
          case 0:
            _context17.next = 2;
            return cmds.callCallbackBound(context, testCallee);

          case 2:
            return _context17.abrupt("return", _context17.sent);

          case 3:
          case "end":
            return _context17.stop();
        }
      }
    }, _marked17, this);
  }
  var actual = await promisify(testCall)();
  var expected = "foobar";
  expect(actual).toEqual(expected);
});

test("callCallbackBound() should handle errors", async function () {
  var _marked18 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var context = {
    message: "bar"
  };
  function testCallee(done) {
    done(new Error("oops"));
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context18) {
      while (1) {
        switch (_context18.prev = _context18.next) {
          case 0:
            _context18.next = 2;
            return cmds.callCallbackBound(context, testCallee);

          case 2:
            return _context18.abrupt("return", _context18.sent);

          case 3:
          case "end":
            return _context18.stop();
        }
      }
    }, _marked18, this);
  }
  try {
    await promisify(testCall)();
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  throw new Error("testCall did not throw");
});

test("callCallbackBound() should handle thrown errors", async function () {
  var _marked19 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var context = {
    message: "bar"
  };
  function testCallee(done) {
    throw new Error("oops");
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context19) {
      while (1) {
        switch (_context19.prev = _context19.next) {
          case 0:
            _context19.next = 2;
            return cmds.callCallbackBound(context, testCallee);

          case 2:
            return _context19.abrupt("return", _context19.sent);

          case 3:
          case "end":
            return _context19.stop();
        }
      }
    }, _marked19, this);
  }
  try {
    await promisify(testCall)();
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  throw new Error("testCall did not throw");
});

test("echo() should return the echoed value", async function () {
  var _marked20 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context20) {
      while (1) {
        switch (_context20.prev = _context20.next) {
          case 0:
            _context20.next = 2;
            return cmds.echo("foobar");

          case 2:
            return _context20.abrupt("return", _context20.sent);

          case 3:
          case "end":
            return _context20.stop();
        }
      }
    }, _marked20, this);
  }
  var actual = await promisify(testCall)();
  var expected = "foobar";
  expect(actual).toEqual(expected);
});

test("noop() should noop", async function () {
  var _marked21 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context21) {
      while (1) {
        switch (_context21.prev = _context21.next) {
          case 0:
            _context21.next = 2;
            return cmds.noop();

          case 2:
            return _context21.abrupt("return", _context21.sent);

          case 3:
          case "end":
            return _context21.stop();
        }
      }
    }, _marked21, this);
  }
  var actual = await promisify(testCall)();
  var expected = undefined;
  expect(actual).toEqual(expected);
});

test("now() should return a timestamp", async function () {
  var _marked22 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var now = Date.now();
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context22) {
      while (1) {
        switch (_context22.prev = _context22.next) {
          case 0:
            _context22.next = 2;
            return cmds.now();

          case 2:
            return _context22.abrupt("return", _context22.sent);

          case 3:
          case "end":
            return _context22.stop();
        }
      }
    }, _marked22, this);
  }
  var actual = await promisify(testCall)();
  expect(actual - now).toBeLessThan(2);
});

test("log() should log to the console", async function () {
  var _marked23 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var logOrig = console.log;
  var actual = void 0;
  console.log = function () {
    for (var _len = arguments.length, values = Array(_len), _key = 0; _key < _len; _key++) {
      values[_key] = arguments[_key];
    }

    actual = values;
  };
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context23) {
      while (1) {
        switch (_context23.prev = _context23.next) {
          case 0:
            _context23.next = 2;
            return cmds.log("foo", "bar");

          case 2:
            return _context23.abrupt("return", _context23.sent);

          case 3:
          case "end":
            return _context23.stop();
        }
      }
    }, _marked23, this);
  }
  await promisify(testCall)();
  console.log = logOrig;
  var expected = ["foo", "bar"];
  expect(actual).toEqual(expected);
});

test("logError() should log to the console", async function () {
  var _marked24 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var logOrig = console.error;
  var actual = void 0;
  console.error = function () {
    for (var _len2 = arguments.length, values = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      values[_key2] = arguments[_key2];
    }

    actual = values;
  };
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context24) {
      while (1) {
        switch (_context24.prev = _context24.next) {
          case 0:
            _context24.next = 2;
            return cmds.logError("foo", "bar");

          case 2:
            return _context24.abrupt("return", _context24.sent);

          case 3:
          case "end":
            return _context24.stop();
        }
      }
    }, _marked24, this);
  }
  await promisify(testCall)();
  console.error = logOrig;
  var expected = ["foo", "bar"];
  expect(actual).toEqual(expected);
});

test("sleep() should sleep", async function () {
  var _marked25 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var start = Date.now();
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context25) {
      while (1) {
        switch (_context25.prev = _context25.next) {
          case 0:
            _context25.next = 2;
            return cmds.sleep(100);

          case 2:
            return _context25.abrupt("return", _context25.sent);

          case 3:
          case "end":
            return _context25.stop();
        }
      }
    }, _marked25, this);
  }
  var actual = await promisify(testCall)();
  var end = Date.now();
  expect(end - start).toBeGreaterThanOrEqual(100);
});

test("series() should execute in series", async function () {
  var _marked26 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var start = Date.now();
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context26) {
      while (1) {
        switch (_context26.prev = _context26.next) {
          case 0:
            _context26.next = 2;
            return cmds.series([cmds.sleep(50), cmds.sleep(50), cmds.sleep(50)]);

          case 2:
            return _context26.abrupt("return", _context26.sent);

          case 3:
          case "end":
            return _context26.stop();
        }
      }
    }, _marked26, this);
  }
  var actual = await promisify(testCall)();
  var end = Date.now();
  expect(end - start).toBeGreaterThanOrEqual(150);
});

test("series() should handle an empty list", async function () {
  var _marked27 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context27) {
      while (1) {
        switch (_context27.prev = _context27.next) {
          case 0:
            _context27.next = 2;
            return cmds.series([]);

          case 2:
            return _context27.abrupt("return", _context27.sent);

          case 3:
          case "end":
            return _context27.stop();
        }
      }
    }, _marked27, this);
  }
  var actual = await promisify(testCall)();
  expect(actual).toEqual([]);
});

test("parellel() should execute in parellel", async function () {
  var _marked28 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var start = Date.now();
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context28) {
      while (1) {
        switch (_context28.prev = _context28.next) {
          case 0:
            _context28.next = 2;
            return cmds.parallel([cmds.sleep(50), cmds.sleep(50), cmds.sleep(50)]);

          case 2:
            return _context28.abrupt("return", _context28.sent);

          case 3:
          case "end":
            return _context28.stop();
        }
      }
    }, _marked28, this);
  }
  var actual = await promisify(testCall)();
  var end = Date.now();
  expect(end - start).toBeGreaterThanOrEqual(50);
  expect(end - start).toBeLessThan(60);
});

test("envelope() should return success results in an envelope", async function () {
  var _marked29 = /*#__PURE__*/regeneratorRuntime.mark(testCallee),
      _marked30 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee(message) {
    return regeneratorRuntime.wrap(function testCallee$(_context29) {
      while (1) {
        switch (_context29.prev = _context29.next) {
          case 0:
            _context29.next = 2;
            return cmds.echo("foo" + message);

          case 2:
            return _context29.abrupt("return", _context29.sent);

          case 3:
          case "end":
            return _context29.stop();
        }
      }
    }, _marked29, this);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context30) {
      while (1) {
        switch (_context30.prev = _context30.next) {
          case 0:
            _context30.next = 2;
            return cmds.envelope(cmds.call(testCallee, "bar"));

          case 2:
            return _context30.abrupt("return", _context30.sent);

          case 3:
          case "end":
            return _context30.stop();
        }
      }
    }, _marked30, this);
  }
  var actual = await promisify(testCall)();
  var expected = { success: true, result: "foobar" };
  expect(actual).toEqual(expected);
});

test("envelope() should return failed results in an envelope", async function () {
  var _marked31 = /*#__PURE__*/regeneratorRuntime.mark(testCallee),
      _marked32 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee(message) {
    return regeneratorRuntime.wrap(function testCallee$(_context31) {
      while (1) {
        switch (_context31.prev = _context31.next) {
          case 0:
            throw new Error("oops");

          case 1:
          case "end":
            return _context31.stop();
        }
      }
    }, _marked31, this);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context32) {
      while (1) {
        switch (_context32.prev = _context32.next) {
          case 0:
            _context32.next = 2;
            return cmds.envelope(cmds.call(testCallee, "bar"));

          case 2:
            return _context32.abrupt("return", _context32.sent);

          case 3:
          case "end":
            return _context32.stop();
        }
      }
    }, _marked32, this);
  }
  var actual = await promisify(testCall)();
  expect(actual.success).toEqual(false);
  expect(actual.result.message).toEqual("oops");
});

test("either() should return value on success", async function () {
  var _marked33 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context33) {
      while (1) {
        switch (_context33.prev = _context33.next) {
          case 0:
            _context33.next = 2;
            return cmds.either(cmds.echo("foobar"), null);

          case 2:
            return _context33.abrupt("return", _context33.sent);

          case 3:
          case "end":
            return _context33.stop();
        }
      }
    }, _marked33, this);
  }
  var actual = await promisify(testCall)();
  var expected = "foobar";
  expect(actual).toEqual(expected);
});

test("either() should return default value on error", async function () {
  var _marked34 = /*#__PURE__*/regeneratorRuntime.mark(testCallee),
      _marked35 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  function testCallee() {
    return regeneratorRuntime.wrap(function testCallee$(_context34) {
      while (1) {
        switch (_context34.prev = _context34.next) {
          case 0:
            throw new Error("oops");

          case 1:
          case "end":
            return _context34.stop();
        }
      }
    }, _marked34, this);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context35) {
      while (1) {
        switch (_context35.prev = _context35.next) {
          case 0:
            _context35.next = 2;
            return cmds.either(cmds.call(testCallee), "defaultvalue");

          case 2:
            return _context35.abrupt("return", _context35.sent);

          case 3:
          case "end":
            return _context35.stop();
        }
      }
    }, _marked35, this);
  }
  var actual = await promisify(testCall)();
  var expected = "defaultvalue";
  expect(actual).toEqual(expected);
});

test('setImmediate() should process a command "immediately"', async function () {
  var _marked36 = /*#__PURE__*/regeneratorRuntime.mark(testCallee),
      _marked37 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var actual = void 0;
  function testCallee(message) {
    return regeneratorRuntime.wrap(function testCallee$(_context36) {
      while (1) {
        switch (_context36.prev = _context36.next) {
          case 0:
            actual = message;

          case 1:
          case "end":
            return _context36.stop();
        }
      }
    }, _marked36, this);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context37) {
      while (1) {
        switch (_context37.prev = _context37.next) {
          case 0:
            _context37.next = 2;
            return cmds.setImmediate(cmds.call(testCallee, "foobar"));

          case 2:
          case "end":
            return _context37.stop();
        }
      }
    }, _marked37, this);
  }
  await promisify(testCall)();
  expect(actual).toEqual(undefined);
  await sleep(10);
  expect(actual).toEqual("foobar");
});

test("setTimeout() should set a timeout", async function () {
  var _marked38 = /*#__PURE__*/regeneratorRuntime.mark(testCallee),
      _marked39 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var actual = void 0;
  function testCallee(message) {
    return regeneratorRuntime.wrap(function testCallee$(_context38) {
      while (1) {
        switch (_context38.prev = _context38.next) {
          case 0:
            actual = message;

          case 1:
          case "end":
            return _context38.stop();
        }
      }
    }, _marked38, this);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context39) {
      while (1) {
        switch (_context39.prev = _context39.next) {
          case 0:
            _context39.next = 2;
            return cmds.setTimeout(cmds.call(testCallee, "foobar"), 10);

          case 2:
          case "end":
            return _context39.stop();
        }
      }
    }, _marked39, this);
  }
  await promisify(testCall)();
  await sleep(5);
  expect(actual).toEqual(undefined);
  await sleep(12);
  expect(actual).toEqual("foobar");
});

test("clearTimeout() should clear a timeout", async function () {
  var _marked40 = /*#__PURE__*/regeneratorRuntime.mark(testCallee),
      _marked41 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var actual = void 0;
  function testCallee(message) {
    return regeneratorRuntime.wrap(function testCallee$(_context40) {
      while (1) {
        switch (_context40.prev = _context40.next) {
          case 0:
            actual = message;

          case 1:
          case "end":
            return _context40.stop();
        }
      }
    }, _marked40, this);
  }
  function testCall() {
    var id;
    return regeneratorRuntime.wrap(function testCall$(_context41) {
      while (1) {
        switch (_context41.prev = _context41.next) {
          case 0:
            _context41.next = 2;
            return cmds.setTimeout(cmds.call(testCallee, "foobar"), 10);

          case 2:
            id = _context41.sent;
            _context41.next = 5;
            return cmds.clearTimeout(id);

          case 5:
          case "end":
            return _context41.stop();
        }
      }
    }, _marked41, this);
  }
  await promisify(testCall)();
  await sleep(5);
  expect(actual).toEqual(undefined);
  await sleep(12);
  expect(actual).toEqual(undefined);
});

test("setInterval() should set an interval", async function () {
  var _marked42 = /*#__PURE__*/regeneratorRuntime.mark(testCallee),
      _marked43 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var actual = 0;
  var id = void 0;
  function testCallee() {
    return regeneratorRuntime.wrap(function testCallee$(_context42) {
      while (1) {
        switch (_context42.prev = _context42.next) {
          case 0:
            actual++;

          case 1:
          case "end":
            return _context42.stop();
        }
      }
    }, _marked42, this);
  }
  function testCall() {
    return regeneratorRuntime.wrap(function testCall$(_context43) {
      while (1) {
        switch (_context43.prev = _context43.next) {
          case 0:
            _context43.next = 2;
            return cmds.setInterval(cmds.call(testCallee), 5);

          case 2:
            id = _context43.sent;

          case 3:
          case "end":
            return _context43.stop();
        }
      }
    }, _marked43, this);
  }
  await promisify(testCall)();
  expect(actual).toEqual(0);
  await sleep(10);
  expect(actual).toBeGreaterThanOrEqual(1);
  await sleep(20);
  expect(actual).toBeGreaterThanOrEqual(3);
  clearInterval(id);
});

test("clearInterval() should set an interval", async function () {
  var _marked44 = /*#__PURE__*/regeneratorRuntime.mark(testCallee),
      _marked45 = /*#__PURE__*/regeneratorRuntime.mark(testCall);

  var actual = 0;
  function testCallee() {
    return regeneratorRuntime.wrap(function testCallee$(_context44) {
      while (1) {
        switch (_context44.prev = _context44.next) {
          case 0:
            actual++;

          case 1:
          case "end":
            return _context44.stop();
        }
      }
    }, _marked44, this);
  }
  function testCall() {
    var id;
    return regeneratorRuntime.wrap(function testCall$(_context45) {
      while (1) {
        switch (_context45.prev = _context45.next) {
          case 0:
            _context45.next = 2;
            return cmds.setInterval(cmds.call(testCallee), 5);

          case 2:
            id = _context45.sent;
            _context45.next = 5;
            return cmds.clearInterval(id);

          case 5:
          case "end":
            return _context45.stop();
        }
      }
    }, _marked45, this);
  }
  await promisify(testCall)();
  expect(actual).toEqual(0);
  await sleep(10);
  expect(actual).toEqual(0);
  await sleep(20);
  expect(actual).toEqual(0);
});

test("getState()/setState() should get and set state", async function () {
  var _marked46 = /*#__PURE__*/regeneratorRuntime.mark(t);

  function t() {
    return regeneratorRuntime.wrap(function t$(_context46) {
      while (1) {
        switch (_context46.prev = _context46.next) {
          case 0:
            _context46.next = 2;
            return cmds.clearState();

          case 2:
            _context46.next = 4;
            return cmds.setState("lorem1", "ipsum1");

          case 4:
            _context46.next = 6;
            return cmds.setState("lorem2", "ipsum2");

          case 6:
            _context46.next = 8;
            return [cmds.getState("lorem1"), cmds.getState("lorem2")];

          case 8:
            return _context46.abrupt("return", _context46.sent);

          case 9:
          case "end":
            return _context46.stop();
        }
      }
    }, _marked46, this);
  }
  var value = await promisify(t)();
  expect(value).toEqual(["ipsum1", "ipsum2"]);
});

test("getState() should return entire state if no key", async function () {
  var _marked47 = /*#__PURE__*/regeneratorRuntime.mark(t);

  function t() {
    return regeneratorRuntime.wrap(function t$(_context47) {
      while (1) {
        switch (_context47.prev = _context47.next) {
          case 0:
            _context47.next = 2;
            return cmds.clearState();

          case 2:
            _context47.next = 4;
            return cmds.setState("dolor", "sit");

          case 4:
            _context47.next = 6;
            return cmds.getState();

          case 6:
            return _context47.abrupt("return", _context47.sent);

          case 7:
          case "end":
            return _context47.stop();
        }
      }
    }, _marked47, this);
  }
  var state = await promisify(t)();
  expect(state).toEqual({ dolor: "sit" });
});

test("getState() should use default value if value is undefined", async function () {
  var _marked48 = /*#__PURE__*/regeneratorRuntime.mark(t);

  function t() {
    return regeneratorRuntime.wrap(function t$(_context48) {
      while (1) {
        switch (_context48.prev = _context48.next) {
          case 0:
            _context48.next = 2;
            return cmds.clearState();

          case 2:
            _context48.next = 4;
            return cmds.getState("amet", "consectetur");

          case 4:
            return _context48.abrupt("return", _context48.sent);

          case 5:
          case "end":
            return _context48.stop();
        }
      }
    }, _marked48, this);
  }
  var value = await promisify(t)();
  expect(value).toEqual("consectetur");
});

test("clearState() should clear the state", async function () {
  var _marked49 = /*#__PURE__*/regeneratorRuntime.mark(t);

  function t() {
    var state;
    return regeneratorRuntime.wrap(function t$(_context49) {
      while (1) {
        switch (_context49.prev = _context49.next) {
          case 0:
            _context49.next = 2;
            return cmds.setState("adipiscing", "elit");

          case 2:
            _context49.next = 4;
            return cmds.getState();

          case 4:
            state = _context49.sent;

            expect(state).toEqual({ adipiscing: "elit" });
            _context49.next = 8;
            return cmds.clearState();

          case 8:
            _context49.next = 10;
            return cmds.getState();

          case 10:
            return _context49.abrupt("return", _context49.sent);

          case 11:
          case "end":
            return _context49.stop();
        }
      }
    }, _marked49, this);
  }
  var state = await promisify(t)();
  expect(state).toEqual({});
});
//# sourceMappingURL=core-cmds-interpreters.spec.js.map