"use strict";

var assert = require("assert");

var _require = require("../core"),
    call = _require.call;

var _require2 = require("./common"),
    interpreters = _require2.interpreters,
    cmds = _require2.cmds,
    functions = _require2.functions;

var _require3 = require("./test-util"),
    expectErrorEquality = _require3.expectErrorEquality;

var basic = functions.basic,
    usesThrowingInterpreter = functions.usesThrowingInterpreter,
    usesRejectingInterpreter = functions.usesRejectingInterpreter;


test("call should reject when a interpreter throws and is not caught", async function () {
  try {
    await call({}, interpreters, usesThrowingInterpreter);
  } catch (actual) {
    var message = "oops";
    return expectErrorEquality(actual, message);
  }
  fail("Function did not reject.");
});

test("call should reject when a interpreter rejects and is not caught", async function () {
  try {
    await call({}, interpreters, usesRejectingInterpreter);
  } catch (actual) {
    var message = "oops";
    return expectErrorEquality(actual, message);
  }
  fail("Function did not reject.");
});

test("call should reject if no context passed in", async function () {
  try {
    await call(null, interpreters, basic);
  } catch (e) {
    assert.equal(e.message, "context required");
    return;
  }
  fail("did not throw");
});

test("call should reject if no function passed in", async function () {
  try {
    await call({}, interpreters);
  } catch (e) {
    assert.equal(e.message, "function required");
    return;
  }
  fail("did not throw");
});
//# sourceMappingURL=core.spec.js.map