"use strict";

var _marked = /*#__PURE__*/regeneratorRuntime.mark(throwInFn),
    _marked2 = /*#__PURE__*/regeneratorRuntime.mark(throwAtYield),
    _marked3 = /*#__PURE__*/regeneratorRuntime.mark(throwAtYieldRecovery);

var _require = require("../core"),
    call = _require.call;

var _require2 = require("./common"),
    interpreters = _require2.interpreters,
    functions = _require2.functions,
    cmds = _require2.cmds;

var usesThrowingInterpreter = functions.usesThrowingInterpreter,
    thrower = functions.thrower;

var _require3 = require("./test-util"),
    expectErrorEquality = _require3.expectErrorEquality;

function throwInFn() {
  return regeneratorRuntime.wrap(function throwInFn$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          throw new Error("oops!");

        case 1:
        case "end":
          return _context.stop();
      }
    }
  }, _marked, this);
}

function throwAtYield() {
  return regeneratorRuntime.wrap(function throwAtYield$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.prev = 0;
          _context2.next = 3;
          return cmds.die();

        case 3:
          _context2.next = 8;
          break;

        case 5:
          _context2.prev = 5;
          _context2.t0 = _context2["catch"](0);
          return _context2.abrupt("return", "caught!");

        case 8:
          return _context2.abrupt("return", "not caught");

        case 9:
        case "end":
          return _context2.stop();
      }
    }
  }, _marked2, this, [[0, 5]]);
}

function throwAtYieldRecovery() {
  return regeneratorRuntime.wrap(function throwAtYieldRecovery$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          _context3.prev = 0;
          _context3.next = 3;
          return cmds.die();

        case 3:
          _context3.next = 7;
          break;

        case 5:
          _context3.prev = 5;
          _context3.t0 = _context3["catch"](0);

        case 7:
          _context3.next = 9;
          return cmds.echo("foo");

        case 9:
          return _context3.abrupt("return", _context3.sent);

        case 10:
        case "end":
          return _context3.stop();
      }
    }
  }, _marked3, this, [[0, 5]]);
}

test("call should catch function errors", async function () {
  try {
    await call({}, interpreters, throwInFn);
  } catch (actual) {
    var message = "oops!";
    return expectErrorEquality(actual, message);
  }
  fail("Function did not reject.");
});

test("call should throw error at the yield", async function () {
  var actual = await call({}, interpreters, throwAtYield);
  var expected = "caught!";
  expect(actual).toEqual(expected);
});

test("call should reject error at the yield and recover", async function () {
  var actual = await call({}, interpreters, throwAtYieldRecovery);
  var expected = "foo";
  expect(actual).toEqual(expected);
});

test("call should reject error for unregistered interpreter", async function () {
  var fn = /*#__PURE__*/regeneratorRuntime.mark(function fn() {
    return regeneratorRuntime.wrap(function fn$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return { type: "dne" };

          case 2:
          case "end":
            return _context4.stop();
        }
      }
    }, fn, this);
  });
  try {
    await call({}, interpreters, fn);
  } catch (e) {
    expect(e.message).toEqual('Interpreter of type "dne" is not registered.');
    return;
  }
  fail("Did not throw.");
});
//# sourceMappingURL=errors.spec.js.map