"use strict";

var assert = require("assert");

var _require = require("../index"),
    promisify = _require.promisify,
    call = _require.call,
    doCmd = _require.doCmd,
    setContext = _require.setContext,
    getContext = _require.getContext,
    addToContext = _require.addToContext,
    setInterpreters = _require.setInterpreters,
    getInterpreters = _require.getInterpreters,
    addInterpreters = _require.addInterpreters,
    reset = _require.reset,
    onError = _require.onError;

var _require2 = require("../index"),
    cmds = _require2.cmds;

test("promisify should tag function", async function () {
  var _marked = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return cmds.echo(message);

          case 2:
            return _context.abrupt("return", _context.sent);

          case 3:
          case "end":
            return _context.stop();
        }
      }
    }, _marked, this);
  }
  expect(promisify(test1).eadPromisified).toEqual(true);
});

test("promisify should name function", async function () {
  var _marked2 = /*#__PURE__*/regeneratorRuntime.mark(testName);

  function testName(message) {
    return regeneratorRuntime.wrap(function testName$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return cmds.echo(message);

          case 2:
            return _context2.abrupt("return", _context2.sent);

          case 3:
          case "end":
            return _context2.stop();
        }
      }
    }, _marked2, this);
  }
  expect(promisify(testName).name).toEqual("testName");
});

test("promisify should not double wrap", async function () {
  var _marked3 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return cmds.echo(message);

          case 2:
            return _context3.abrupt("return", _context3.sent);

          case 3:
          case "end":
            return _context3.stop();
        }
      }
    }, _marked3, this);
  }
  var p = promisify(test1);
  expect(p).toEqual(promisify(p));
});

test("promisify should wrap up an effects-as-data function", async function () {
  var _marked4 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return cmds.echo(message);

          case 2:
            return _context4.abrupt("return", _context4.sent);

          case 3:
          case "end":
            return _context4.stop();
        }
      }
    }, _marked4, this);
  }

  var result = await promisify(test1)("foo");
  expect(result).toEqual("foo");
});

test("promisify should handle errors", async function () {
  var _marked5 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            throw new Error("oops");

          case 1:
          case "end":
            return _context5.stop();
        }
      }
    }, _marked5, this);
  }

  try {
    await promisify(test1)("foo");
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  fail("promisify did not reject");
});

test("call should call a function", async function () {
  var _marked6 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return cmds.echo(message);

          case 2:
            return _context6.abrupt("return", _context6.sent);

          case 3:
          case "end":
            return _context6.stop();
        }
      }
    }, _marked6, this);
  }

  var result = await call(test1, "foo");
  expect(result).toEqual("foo");
});

test("doCmd should process a command", async function () {
  var result = await doCmd(cmds.echo("foo"));
  expect(result).toEqual("foo");
});

test("cmds.call should call effects-as-data function", async function () {
  var _marked7 = /*#__PURE__*/regeneratorRuntime.mark(test1),
      _marked8 = /*#__PURE__*/regeneratorRuntime.mark(test2);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            _context7.next = 2;
            return cmds.call(test2, message);

          case 2:
            return _context7.abrupt("return", _context7.sent);

          case 3:
          case "end":
            return _context7.stop();
        }
      }
    }, _marked7, this);
  }

  function test2(message) {
    return regeneratorRuntime.wrap(function test2$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            _context8.next = 2;
            return cmds.echo(message);

          case 2:
            return _context8.abrupt("return", _context8.sent);

          case 3:
          case "end":
            return _context8.stop();
        }
      }
    }, _marked8, this);
  }

  var result = await promisify(test1)("foo");
  expect(result).toEqual("foo");
});

test("[error handling] cmds.call should call effects-as-data function", async function () {
  var _marked9 = /*#__PURE__*/regeneratorRuntime.mark(test1),
      _marked10 = /*#__PURE__*/regeneratorRuntime.mark(test2);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            _context9.next = 2;
            return cmds.call(test2, message);

          case 2:
            return _context9.abrupt("return", _context9.sent);

          case 3:
          case "end":
            return _context9.stop();
        }
      }
    }, _marked9, this);
  }

  function test2(message) {
    return regeneratorRuntime.wrap(function test2$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            throw new Error("oops");

          case 1:
          case "end":
            return _context10.stop();
        }
      }
    }, _marked10, this);
  }

  try {
    await promisify(test1)("foo");
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  fail("promisify did not reject");
});

test("cmds.call should call promisified function", async function () {
  var _marked11 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            _context11.next = 2;
            return cmds.call(test2, message);

          case 2:
            return _context11.abrupt("return", _context11.sent);

          case 3:
          case "end":
            return _context11.stop();
        }
      }
    }, _marked11, this);
  }

  var test2 = promisify( /*#__PURE__*/regeneratorRuntime.mark(function _callee(message) {
    return regeneratorRuntime.wrap(function _callee$(_context12) {
      while (1) {
        switch (_context12.prev = _context12.next) {
          case 0:
            _context12.next = 2;
            return cmds.echo(message);

          case 2:
            return _context12.abrupt("return", _context12.sent);

          case 3:
          case "end":
            return _context12.stop();
        }
      }
    }, _callee, this);
  }));

  var result = await promisify(test1)("foo");
  expect(result).toEqual("foo");
});

test("[error handling] cmds.call should call promisified function", async function () {
  var _marked12 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            _context13.next = 2;
            return cmds.call(test2, message);

          case 2:
            return _context13.abrupt("return", _context13.sent);

          case 3:
          case "end":
            return _context13.stop();
        }
      }
    }, _marked12, this);
  }

  var test2 = promisify( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(message) {
    return regeneratorRuntime.wrap(function _callee2$(_context14) {
      while (1) {
        switch (_context14.prev = _context14.next) {
          case 0:
            throw new Error("oops");

          case 1:
          case "end":
            return _context14.stop();
        }
      }
    }, _callee2, this);
  }));

  try {
    await promisify(test1)("foo");
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  fail("promisify did not reject");
});

test("cmds.call.fn should call promisified function", async function () {
  var _marked13 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context15) {
      while (1) {
        switch (_context15.prev = _context15.next) {
          case 0:
            _context15.next = 2;
            return cmds.call.fn(test2, message);

          case 2:
            return _context15.abrupt("return", _context15.sent);

          case 3:
          case "end":
            return _context15.stop();
        }
      }
    }, _marked13, this);
  }

  var test2 = promisify( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(message) {
    return regeneratorRuntime.wrap(function _callee3$(_context16) {
      while (1) {
        switch (_context16.prev = _context16.next) {
          case 0:
            _context16.next = 2;
            return cmds.echo(message);

          case 2:
            return _context16.abrupt("return", _context16.sent);

          case 3:
          case "end":
            return _context16.stop();
        }
      }
    }, _callee3, this);
  }));

  var result = await promisify(test1)("foo");
  expect(result).toEqual("foo");
});

test("[error handling] cmds.call.fn should call promisified function", async function () {
  var _marked14 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context17) {
      while (1) {
        switch (_context17.prev = _context17.next) {
          case 0:
            _context17.next = 2;
            return cmds.call.fn(test2, message);

          case 2:
            return _context17.abrupt("return", _context17.sent);

          case 3:
          case "end":
            return _context17.stop();
        }
      }
    }, _marked14, this);
  }

  var test2 = promisify( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(message) {
    return regeneratorRuntime.wrap(function _callee4$(_context18) {
      while (1) {
        switch (_context18.prev = _context18.next) {
          case 0:
            throw new Error("oops");

          case 1:
          case "end":
            return _context18.stop();
        }
      }
    }, _callee4, this);
  }));

  try {
    await promisify(test1)("foo");
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  fail("promisify did not reject");
});

test("cmds.call.fn should call promise returning function", async function () {
  var _marked15 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context19) {
      while (1) {
        switch (_context19.prev = _context19.next) {
          case 0:
            _context19.next = 2;
            return cmds.call.fn(test2, message);

          case 2:
            return _context19.abrupt("return", _context19.sent);

          case 3:
          case "end":
            return _context19.stop();
        }
      }
    }, _marked15, this);
  }

  function test2(message) {
    return Promise.resolve(message);
  }

  var result = await promisify(test1)("foo");
  expect(result).toEqual("foo");
});

test("[error handling] cmds.call.fn should call promise returning function", async function () {
  var _marked16 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context20) {
      while (1) {
        switch (_context20.prev = _context20.next) {
          case 0:
            _context20.next = 2;
            return cmds.call.fn(test2, message);

          case 2:
            return _context20.abrupt("return", _context20.sent);

          case 3:
          case "end":
            return _context20.stop();
        }
      }
    }, _marked16, this);
  }

  function test2(message) {
    throw new Error("oops");
  }

  try {
    await promisify(test1)("foo");
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  fail("promisify did not reject");
});

test("cmds.call.fnBound should call promise returning function", async function () {
  var _marked17 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context21) {
      while (1) {
        switch (_context21.prev = _context21.next) {
          case 0:
            _context21.next = 2;
            return cmds.call.fnBound(obj, obj.test2, message);

          case 2:
            return _context21.abrupt("return", _context21.sent);

          case 3:
          case "end":
            return _context21.stop();
        }
      }
    }, _marked17, this);
  }

  var obj = {
    value: "bar",
    test2: function test2(message) {
      return Promise.resolve("" + message + this.value);
    }
  };

  var result = await promisify(test1)("foo");
  expect(result).toEqual("foobar");
});

test("[error handling] cmds.call.fnBound should call promise returning function", async function () {
  var _marked18 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context22) {
      while (1) {
        switch (_context22.prev = _context22.next) {
          case 0:
            _context22.next = 2;
            return cmds.call.fnBound(obj, obj.test2, message);

          case 2:
            return _context22.abrupt("return", _context22.sent);

          case 3:
          case "end":
            return _context22.stop();
        }
      }
    }, _marked18, this);
  }

  var obj = {
    value: "bar",
    test2: function test2(message) {
      throw new Error("oops");
    }
  };

  try {
    await promisify(test1)("foo");
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  fail("promisify did not reject");
});

test("cmds.call.callback should call callback function", async function () {
  var _marked19 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context23) {
      while (1) {
        switch (_context23.prev = _context23.next) {
          case 0:
            _context23.next = 2;
            return cmds.call.callback(test2, message);

          case 2:
            return _context23.abrupt("return", _context23.sent);

          case 3:
          case "end":
            return _context23.stop();
        }
      }
    }, _marked19, this);
  }

  function test2(message, done) {
    done(null, message);
  }

  var result = await promisify(test1)("foo");
  expect(result).toEqual("foo");
});

test("[error handling] cmds.call.callback should call callback function", async function () {
  var _marked20 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context24) {
      while (1) {
        switch (_context24.prev = _context24.next) {
          case 0:
            _context24.next = 2;
            return cmds.call.callback(test2, message);

          case 2:
            return _context24.abrupt("return", _context24.sent);

          case 3:
          case "end":
            return _context24.stop();
        }
      }
    }, _marked20, this);
  }

  function test2(message, done) {
    done(new Error("oops"), message);
  }

  try {
    await promisify(test1)("foo");
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  fail("promisify did not reject");
});

test("[error handling 2] cmds.call.callback should call callback function", async function () {
  var _marked21 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context25) {
      while (1) {
        switch (_context25.prev = _context25.next) {
          case 0:
            _context25.next = 2;
            return cmds.call.callback(test2, message);

          case 2:
            return _context25.abrupt("return", _context25.sent);

          case 3:
          case "end":
            return _context25.stop();
        }
      }
    }, _marked21, this);
  }

  function test2(message, done) {
    throw new Error("oops");
  }

  try {
    await promisify(test1)("foo");
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  fail("promisify did not reject");
});

test("cmds.call.callbackBound should call callback function", async function () {
  var _marked22 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context26) {
      while (1) {
        switch (_context26.prev = _context26.next) {
          case 0:
            _context26.next = 2;
            return cmds.call.callbackBound(obj, obj.test2, message);

          case 2:
            return _context26.abrupt("return", _context26.sent);

          case 3:
          case "end":
            return _context26.stop();
        }
      }
    }, _marked22, this);
  }

  var obj = {
    value: "bar",
    test2: function test2(message, done) {
      done(null, "" + message + this.value);
    }
  };

  var result = await promisify(test1)("foo");
  expect(result).toEqual("foobar");
});

test("[error handling] cmds.call.callbackBound should call callback function", async function () {
  var _marked23 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context27) {
      while (1) {
        switch (_context27.prev = _context27.next) {
          case 0:
            _context27.next = 2;
            return cmds.call.callbackBound(obj, obj.test2, message);

          case 2:
            return _context27.abrupt("return", _context27.sent);

          case 3:
          case "end":
            return _context27.stop();
        }
      }
    }, _marked23, this);
  }

  var obj = {
    value: "bar",
    test2: function test2(message, done) {
      done(new Error("oops"), "" + message + this.value);
    }
  };

  try {
    await promisify(test1)("foo");
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  fail("promisify did not reject");
});

test("[error handling] cmds.call.callbackBound should call callback function", async function () {
  var _marked24 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  function test1(message) {
    return regeneratorRuntime.wrap(function test1$(_context28) {
      while (1) {
        switch (_context28.prev = _context28.next) {
          case 0:
            _context28.next = 2;
            return cmds.call.callbackBound(obj, obj.test2, message);

          case 2:
            return _context28.abrupt("return", _context28.sent);

          case 3:
          case "end":
            return _context28.stop();
        }
      }
    }, _marked24, this);
  }

  var obj = {
    value: "bar",
    test2: function test2(message, done) {
      throw new Error("oops");
    }
  };

  try {
    await promisify(test1)("foo");
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  fail("promisify did not reject");
});

test("promisify(fn).callWithContext({...}) should call function with context patch", async function () {
  var _marked25 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  setContext({ configProperty: "foo" });
  // Interpreter
  function testEffect(cmd, _ref) {
    var context = _ref.context;

    return context.configProperty;
  }

  addInterpreters({ testEffect: testEffect });

  function test1(message) {
    var result;
    return regeneratorRuntime.wrap(function test1$(_context29) {
      while (1) {
        switch (_context29.prev = _context29.next) {
          case 0:
            _context29.next = 2;
            return { type: "testEffect" };

          case 2:
            result = _context29.sent;
            return _context29.abrupt("return", "" + result + message);

          case 4:
          case "end":
            return _context29.stop();
        }
      }
    }, _marked25, this);
  }

  var result1 = await promisify(test1)("bar");
  expect(result1).toEqual("foobar");

  var result2 = await promisify(test1).callWithContext({
    configProperty: "bar"
  }, "bar");
  expect(result2).toEqual("barbar");
});

test("promisify(fn).callWithContext({...}) should call function with context patch and call validator", async function () {
  var _marked26 = /*#__PURE__*/regeneratorRuntime.mark(test1);

  setContext({ configProperty: "foo" });
  // Interpreter
  function testEffect(cmd, _ref2) {
    var context = _ref2.context;

    return context.configProperty;
  }

  addInterpreters({ testEffect: testEffect });

  function test1(message) {
    var result;
    return regeneratorRuntime.wrap(function test1$(_context30) {
      while (1) {
        switch (_context30.prev = _context30.next) {
          case 0:
            _context30.next = 2;
            return { type: "testEffect" };

          case 2:
            result = _context30.sent;
            return _context30.abrupt("return", "" + result + message);

          case 4:
          case "end":
            return _context30.stop();
        }
      }
    }, _marked26, this);
  }

  test1.validator = function () {
    throw new Error("not valid");
  };

  try {
    await promisify(test1).callWithContext({
      configProperty: "bar"
    }, "bar");
  } catch (e) {
    assert.equal(e.message, "not valid");
    return;
  }
  fail("did not throw");
});

test("addInterpreters(), getInterpreters(), setInterpreters()", function () {
  reset();

  function testEffect1() {
    return "test1";
  }

  function testEffect2() {
    return "test2";
  }

  expect(getInterpreters()).toEqual({});

  addInterpreters({ testEffect1: testEffect1 });
  expect(getInterpreters()).toEqual({ testEffect1: testEffect1 });

  addInterpreters({ testEffect2: testEffect2 });
  expect(getInterpreters()).toEqual({ testEffect1: testEffect1, testEffect2: testEffect2 });

  setInterpreters({});
  expect(getInterpreters()).toEqual({});
});

test("addToContext(), getContext(), setContext()", function () {
  reset();

  expect(getContext()).toEqual({});

  addToContext({ test1: "foo" });
  expect(getContext()).toEqual({ test1: "foo" });

  addToContext({ test2: "bar" });
  expect(getContext()).toEqual({ test1: "foo", test2: "bar" });

  setContext({});
  expect(getContext()).toEqual({});
});

test("onError() should register error handler on context", function () {
  var handler = function handler() {};
  onError(handler);
  var context = getContext();
  expect(context.onError).toEqual(handler);
});

test("onError() should throw if handler is not a function", function () {
  expect(onError).toThrow("onError requires a function");
});
//# sourceMappingURL=index.spec.js.map