"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _marked = /*#__PURE__*/regeneratorRuntime.mark(basic),
    _marked2 = /*#__PURE__*/regeneratorRuntime.mark(basicMultistep);

var _require = require("../core"),
    call = _require.call,
    buildFunctions = _require.buildFunctions;

var _require2 = require("./common"),
    interpreters = _require2.interpreters,
    functions = _require2.functions,
    cmds = _require2.cmds;

var usesThrowingInterpreter = functions.usesThrowingInterpreter;

var _require3 = require("./test-util"),
    sleep = _require3.sleep;

var coreInterpreters = require("../interpreters");
var coreCmds = require("../cmds");

function basic(message) {
  return regeneratorRuntime.wrap(function basic$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return cmds.echo(message);

        case 2:
          return _context.abrupt("return", _context.sent);

        case 3:
        case "end":
          return _context.stop();
      }
    }
  }, _marked, this);
}

function basicMultistep(message) {
  var s1, s2;
  return regeneratorRuntime.wrap(function basicMultistep$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return cmds.echo(message + "1");

        case 2:
          s1 = _context2.sent;
          _context2.next = 5;
          return cmds.echo(message + "2");

        case 5:
          s2 = _context2.sent;
          return _context2.abrupt("return", { s1: s1, s2: s2 });

        case 7:
        case "end":
          return _context2.stop();
      }
    }
  }, _marked2, this);
}

test("should add a stack to the context and push the current frame", async function () {
  var telemetry = void 0;
  var onCommand = function onCommand(t) {
    telemetry = t;
  };
  var context = { onCommand: onCommand, name: "telemetry" };
  var now = Date.now();
  await call(context, interpreters, basic, "foo");
  await sleep(10);
  expect(telemetry.context.stack[0].fn).toEqual(basic);
  expect(telemetry.context.stack[0].interpreters).toEqual(interpreters);
  expect(telemetry.context.stack[0].args).toEqual(["foo"]);
  expect(telemetry.context.stack[0].context.onCommand).toEqual(onCommand);
  expect(telemetry.context.stack[0].context.name).toEqual("telemetry");
  expect(telemetry.context.stack[0].context.stack).toEqual(undefined);

  // Should be serializable
  JSON.stringify(telemetry);
});

test("should add a stack to the context for child calls", async function () {
  var telemetry = void 0;
  var onCommand = function onCommand(t) {
    telemetry = t;
  };
  var context = { onCommand: onCommand, name: "telemetry" };
  var now = Date.now();
  await call(context, interpreters, basic, "foo");
  await sleep(10);
  expect(telemetry.context.stack[0].fn).toEqual(basic);
  expect(telemetry.context.stack[0].interpreters).toEqual(interpreters);
  expect(telemetry.context.stack[0].args).toEqual(["foo"]);
  expect(telemetry.context.stack[0].context.onCommand).toEqual(onCommand);
  expect(telemetry.context.stack[0].context.name).toEqual("telemetry");
  expect(telemetry.context.stack[0].context.stack).toEqual(undefined);

  // Should be serializable
  JSON.stringify(telemetry);
});

test("onCommand", async function () {
  var telemetry = [];
  var onCommand = function onCommand(t) {
    telemetry.push(t);
  };
  var context = { onCommand: onCommand, name: "telemetry" };
  var now = Date.now();
  await call(context, interpreters, basicMultistep, "foo");
  await sleep(10);
  expect(telemetry.length).toEqual(2);
  telemetry.forEach(function (t, i) {
    var message = "foo" + (i + 1);
    expect(t.command).toEqual(cmds.echo(message));
    expect(t.fn).toEqual(basicMultistep);
    expect(t.start).toBeGreaterThanOrEqual(now);
    expect(t.index).toEqual(0);
    expect(t.step).toEqual(i);
    expect(t.context.name).toEqual("telemetry");
    expect(t.context.stack[0].fn).toEqual(basicMultistep);
  });

  // Should be serializable
  JSON.stringify(telemetry);
});

test("onCommandComplete", async function () {
  var telemetry = [];
  var onCommandComplete = function onCommandComplete(t) {
    telemetry.push(t);
  };
  var context = { onCommandComplete: onCommandComplete, name: "telemetry" };
  var now = Date.now();
  await call(context, interpreters, basicMultistep, "foo");
  await sleep(10);
  expect(telemetry.length).toEqual(2);
  telemetry.forEach(function (t, i) {
    var message = "foo" + (i + 1);
    expect(t.success).toEqual(true);
    expect(t.command).toEqual(cmds.echo(message));
    expect(t.latency).toBeLessThan(5);
    expect(t.start).toBeGreaterThanOrEqual(now);
    expect(t.end).toBeGreaterThanOrEqual(t.start);
    expect(t.index).toEqual(0);
    expect(t.step).toEqual(i);
    expect(t.result).toEqual(message);
    expect(t.context.name).toEqual("telemetry");
    expect(t.context.stack[0].fn).toEqual(basicMultistep);
    expect(t.fn).toEqual(basicMultistep);
  });

  // Should be serializable
  JSON.stringify(telemetry);
});

test("telemetry on error - onCommandComplete", async function () {
  var telemetry = void 0;
  var onCommandComplete = function onCommandComplete(t) {
    telemetry = t;
  };
  var context = { onCommandComplete: onCommandComplete, name: "usesThrowingInterpreter" };
  var now = Date.now();
  var message = "oops";
  try {
    await call(context, interpreters, usesThrowingInterpreter, message);
  } catch (e) {}
  await sleep(10);
  expect(telemetry.success).toEqual(false);
  expect(telemetry.command).toEqual(cmds.die(message));
  expect(telemetry.latency).toBeLessThan(5);
  expect(telemetry.start).toBeGreaterThanOrEqual(now);
  expect(telemetry.end).toBeGreaterThanOrEqual(telemetry.start);
  expect(telemetry.index).toEqual(0);
  expect(telemetry.step).toEqual(0);
  expect(telemetry.result.message).toEqual("oops");
  expect(telemetry.context.name).toEqual("usesThrowingInterpreter");
  expect(telemetry.context.stack[0].fn).toEqual(usesThrowingInterpreter);
  expect(telemetry.fn).toEqual(usesThrowingInterpreter);

  // Should be serializable
  JSON.stringify(telemetry);
});

test("onCall", function (done) {
  var onCall = function onCall(called) {
    expect(called.args).toEqual(["foo", "bar", "baz"]);
    done();
  };
  var context = { onCall: onCall };
  call(context, interpreters, basicMultistep, "foo", "bar", "baz");
});

test("onCallComplete", function (done) {
  var now = Date.now();
  var onCallComplete = function onCallComplete(complete) {
    expect(complete.success).toEqual(true);
    expect(complete.fn).toEqual(basic);
    expect(complete.result).toEqual("foo");
    expect(_typeof(complete.latency)).toEqual("number");
    expect(complete.start).toBeGreaterThanOrEqual(now);
    expect(complete.end).toBeGreaterThanOrEqual(complete.start);
    done();
  };
  var context = { onCallComplete: onCallComplete, name: "telemetry" };
  call(context, interpreters, basic, "foo");
});

test("onCallComplete for errors from interpreters", function (done) {
  var now = Date.now();
  var onCallComplete = function onCallComplete(complete) {
    expect(complete.success).toEqual(false);
    expect(complete.fn).toEqual(usesThrowingInterpreter);
    expect(complete.result.message).toEqual("oops");
    expect(_typeof(complete.latency)).toEqual("number");
    expect(complete.start).toBeGreaterThanOrEqual(now);
    expect(complete.end).toBeGreaterThanOrEqual(complete.start);
    done();
  };
  var context = { onCallComplete: onCallComplete, name: "telemetry" };
  call(context, interpreters, usesThrowingInterpreter, "foo").catch(function (e) {
    return e;
  });
});

test("onCallComplete for errors from function body", function (done) {
  var _marked3 = /*#__PURE__*/regeneratorRuntime.mark(throwFromBody);

  var now = Date.now();
  var onCallComplete = function onCallComplete(complete) {
    expect(complete.success).toEqual(false);
    expect(complete.fn).toEqual(throwFromBody);
    expect(complete.result.message).toEqual("oops");
    expect(_typeof(complete.latency)).toEqual("number");
    expect(complete.start).toBeGreaterThanOrEqual(now);
    expect(complete.end).toBeGreaterThanOrEqual(complete.start);
    done();
  };
  function throwFromBody() {
    return regeneratorRuntime.wrap(function throwFromBody$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            throw new Error("oops");

          case 1:
          case "end":
            return _context3.stop();
        }
      }
    }, _marked3, this);
  }
  var context = { onCallComplete: onCallComplete, name: "telemetry" };
  call(context, interpreters, throwFromBody).catch(function (e) {
    return e;
  });
});

test("onCallComplete for errors from function body when using buildFunctions", function (done) {
  var _marked4 = /*#__PURE__*/regeneratorRuntime.mark(throwFromBody);

  var now = Date.now();
  var onCallComplete = function onCallComplete(complete) {
    expect(complete.success).toEqual(false);
    expect(complete.fn).toEqual(throwFromBody);
    expect(complete.result.message).toEqual("oops");
    expect(_typeof(complete.latency)).toEqual("number");
    expect(complete.start).toBeGreaterThanOrEqual(now);
    expect(complete.end).toBeGreaterThanOrEqual(complete.start);
    done();
  };
  function throwFromBody() {
    return regeneratorRuntime.wrap(function throwFromBody$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return cmds.echo("foo");

          case 2:
            throw new Error("oops");

          case 3:
          case "end":
            return _context4.stop();
        }
      }
    }, _marked4, this);
  }
  var context = { onCallComplete: onCallComplete, name: "telemetry" };
  var built = buildFunctions(context, interpreters, { throwFromBody: throwFromBody });
  built.throwFromBody().catch(function (e) {
    return e;
  });
});

test("telemetry onError - interpreter", async function () {
  var error = void 0;
  var errorContext = void 0;
  var callCount = 0;
  var onError = function onError(e, c) {
    error = e;
    errorContext = c;
    callCount++;
  };
  var context = { onError: onError, name: "usesThrowingInterpreter" };
  var now = Date.now();
  var message = "oops";
  try {
    await call(context, interpreters, usesThrowingInterpreter, message);
  } catch (e) {}
  await sleep(10);
  expect(error.message).toEqual("oops");
  expect(errorContext.name).toEqual("usesThrowingInterpreter");
  expect(errorContext.stack[0].fn).toEqual(usesThrowingInterpreter);
  expect(errorContext.stack[0].args).toEqual([message]);
  expect(callCount).toEqual(1);

  // Should be serializable
  JSON.stringify(error);
});

test("telemetry onError - function body", async function () {
  var _marked5 = /*#__PURE__*/regeneratorRuntime.mark(throwFromBody);

  var error = void 0;
  var errorContext = void 0;
  var callCount = 0;
  var onError = function onError(e, c) {
    error = e;
    errorContext = c;
    callCount++;
  };
  var context = { onError: onError, name: "throwFromBody" };
  var now = Date.now();
  function throwFromBody() {
    return regeneratorRuntime.wrap(function throwFromBody$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return cmds.echo("foo");

          case 2:
            throw new Error("oops");

          case 3:
          case "end":
            return _context5.stop();
        }
      }
    }, _marked5, this);
  }
  try {
    await call(context, interpreters, throwFromBody, "arg1", "arg2");
  } catch (e) {}
  await sleep(10);
  expect(error.message).toEqual("oops");
  expect(errorContext.name).toEqual("throwFromBody");
  expect(errorContext.stack[0].fn).toEqual(throwFromBody);
  expect(errorContext.stack[0].args).toEqual(["arg1", "arg2"]);
  expect(callCount).toEqual(1);

  // Should be serializable
  JSON.stringify(error);
});

test("should not mutate stack", async function () {
  var _marked6 = /*#__PURE__*/regeneratorRuntime.mark(nestedCall);

  var stack1 = void 0;
  var stack2 = void 0;
  var onCommand = function onCommand(t) {
    if (!stack1) {
      stack1 = t.context.stack;
      return;
    }
    if (!stack2) {
      stack2 = t.context.stack;
      return;
    }
  };
  function nestedCall() {
    return regeneratorRuntime.wrap(function nestedCall$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            if (!stack2) {
              _context6.next = 2;
              break;
            }

            return _context6.abrupt("return");

          case 2:
            _context6.next = 4;
            return coreCmds.setImmediate(nestedCall);

          case 4:
          case "end":
            return _context6.stop();
        }
      }
    }, _marked6, this);
  }
  var context = { onCommand: onCommand, name: "telemetry" };
  var now = Date.now();
  await call(context, coreInterpreters, nestedCall, "foo");
  await sleep(10);
  expect(stack1).toBeTruthy();
  expect(stack2).toBeTruthy();
  expect(stack1).not.toBe(stack2);
});
//# sourceMappingURL=telemetry.spec.js.map