"use strict";

var _marked = /*#__PURE__*/regeneratorRuntime.mark(singleLine),
    _marked2 = /*#__PURE__*/regeneratorRuntime.mark(yieldArray),
    _marked3 = /*#__PURE__*/regeneratorRuntime.mark(rethrow),
    _marked4 = /*#__PURE__*/regeneratorRuntime.mark(throwFoo),
    _marked5 = /*#__PURE__*/regeneratorRuntime.mark(throwImmediately),
    _marked6 = /*#__PURE__*/regeneratorRuntime.mark(returnCmdResult),
    _marked7 = /*#__PURE__*/regeneratorRuntime.mark(throwSemantic);

var _require = require("./common"),
    functions = _require.functions,
    cmds = _require.cmds;

var basic = functions.basic,
    basicMultistep = functions.basicMultistep,
    basicParallel = functions.basicParallel,
    basicMultistepParallel = functions.basicMultistepParallel,
    basicEmpty = functions.basicEmpty,
    basicMultiArg = functions.basicMultiArg,
    usesThrowingInterpreter = functions.usesThrowingInterpreter;

var _require2 = require("../test"),
    testFn = _require2.testFn,
    testFnV2 = _require2.testFnV2,
    args = _require2.args;

var _require3 = require("./test-util"),
    deepEqual = _require3.deepEqual;

var _require4 = require("../index"),
    promisify = _require4.promisify;

function singleLine(id) {
  var s1;
  return regeneratorRuntime.wrap(function singleLine$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return cmds.httpGet("http://example.com/api/v1/users/" + id);

        case 2:
          s1 = _context.sent;
          return _context.abrupt("return", s1);

        case 4:
        case "end":
          return _context.stop();
      }
    }
  }, _marked, this);
}

function yieldArray() {
  var s1;
  return regeneratorRuntime.wrap(function yieldArray$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return [{ type: "test" }];

        case 2:
          s1 = _context2.sent;
          return _context2.abrupt("return", s1);

        case 4:
        case "end":
          return _context2.stop();
      }
    }
  }, _marked2, this);
}

var testSingleLine = testFn(singleLine);
var testYieldArray = testFn(yieldArray);

function rethrow() {
  return regeneratorRuntime.wrap(function rethrow$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          _context3.prev = 0;
          _context3.next = 3;
          return cmds.echo("foo");

        case 3:
          return _context3.abrupt("return", _context3.sent);

        case 6:
          _context3.prev = 6;
          _context3.t0 = _context3["catch"](0);
          throw new Error("bar");

        case 9:
        case "end":
          return _context3.stop();
      }
    }
  }, _marked3, this, [[0, 6]]);
}

test("testFn should fail if tuples[0][0] is not an array", function () {
  try {
    testFn(throwFoo, function () {
      // prettier-ignore
      return [[null, new Error("bar")]];
    })();
  } catch (e) {
    deepEqual(e.message, "The first argument of the first tuple must be an array representing the arguments of the function.");
    return;
  }
  throw new Error("Failed: Did not compare error messages");
});

function throwFoo() {
  return regeneratorRuntime.wrap(function throwFoo$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          throw new Error("foo");

        case 1:
        case "end":
          return _context4.stop();
      }
    }
  }, _marked4, this);
}

test("testFn should fail if the function error is different than the test error", function () {
  try {
    testFn(throwFoo, function () {
      // prettier-ignore
      return [[[], new Error("bar")]];
    })();
  } catch (e) {
    deepEqual(e.stack.includes("Error on Step 1"), true);
    return;
  }
  throw new Error("Failed: Did not compare error messages");
});

test("should be able to rethrow errors", testFnV2(rethrow, function () {
  // prettier-ignore
  return [[], [cmds.echo('foo'), new Error("whatever")], new Error("bar")];
}));

test("testFn should handle undefined returns for semantic test", testFn( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
  return regeneratorRuntime.wrap(function _callee$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
        case "end":
          return _context5.stop();
      }
    }
  }, _callee, this);
}), function () {
  return args().returns();
}));

test("testFn should curry", testFn(basic)(function () {
  // prettier-ignore
  return [[['foo'], cmds.echo('foo')], ['foo', 'foo']];
}));

var basicPromisified = promisify(basic);
test("testFn should handle promisified functions", testFn(basicPromisified)(function () {
  // prettier-ignore
  return [[['foo'], cmds.echo('foo')], ['foo', 'foo']];
}));

test("testFn should pass (basic)", testFn(basic, function () {
  // prettier-ignore
  return [[['foo'], cmds.echo('foo')], ['foo', 'foo']];
}));

// Basic

test("testFn semantic should pass (basic)", testFn(basic, function () {
  // prettier-ignore
  return args('foo').cmd(cmds.echo('foo')).result('foo').returns('foo');
}));

test("testFnV2 should pass (basic)", testFnV2(basic, function () {
  // prettier-ignore
  return [['foo'], [cmds.echo('foo'), 'foo'], 'foo'];
}));

test("testFn should pass (basicMultiArg)", testFn(basicMultiArg, function () {
  // prettier-ignore
  return [[['foo', 'bar'], cmds.echo('foobar')], ['foobar', 'foobar']];
}));

// Basic w multiple arguments

test("testFn should pass (basicMultiArg)", testFn(basicMultiArg, function () {
  // prettier-ignore
  return [[['foo', 'bar'], cmds.echo('foobar')], ['foobar', 'foobar']];
}));

test("testFn semantic should pass (basicMultiArg)", testFn(basicMultiArg, function () {
  // prettier-ignore
  return args('foo', 'bar').cmd(cmds.echo('foobar')).result('foobar').returns('foobar');
}));

test("testFn semantic should pass (basicMultiArg)", testFnV2(basicMultiArg, function () {
  // prettier-ignore
  return [['foo', 'bar'], [cmds.echo('foobar'), 'foobar'], 'foobar'];
}));

// Basic with multiple steps

test("testFn should pass (basicMultistep)", testFn(basicMultistep, function () {
  // prettier-ignore
  return [[['foo'], cmds.echo('foo1')], ['foo1', cmds.echo('foo2')], ['foo2', { s1: 'foo1', s2: 'foo2' }]];
}));

test("testFn semantic should pass (basicMultistep)", testFn(basicMultistep, function () {
  // prettier-ignore
  return args('foo').cmd(cmds.echo('foo1')).result('foo1').cmd(cmds.echo('foo2')).result('foo2').returns({ s1: 'foo1', s2: 'foo2' });
}));

test("testFnV2 should pass (basicMultistep)", testFnV2(basicMultistep, function () {
  // prettier-ignore
  return [['foo'], [cmds.echo('foo1'), 'foo1'], [cmds.echo('foo2'), 'foo2'], { s1: 'foo1', s2: 'foo2' }];
}));

// Basic with parallel commands

test("testFn should pass (basicParallel)", testFn(basicParallel, function () {
  var c = [cmds.echo("foo"), cmds.echo("foo")];
  // prettier-ignore
  return [[['foo'], c], [['foo', 'foo'], { s1: 'foo1', s2: 'foo2' }]];
}));

test("testFn semantic should pass (basicParallel)", testFn(basicParallel, function () {
  var c = [cmds.echo("foo"), cmds.echo("foo")];
  // prettier-ignore
  return args('foo').cmd(c).result(['foo', 'foo']).returns({ s1: 'foo1', s2: 'foo2' });
}));

test("testFnV2 should pass (basicParallel)", testFnV2(basicParallel, function () {
  var c = [cmds.echo("foo"), cmds.echo("foo")];
  // prettier-ignore
  return [['foo'], [c, ['foo', 'foo']], { s1: 'foo1', s2: 'foo2' }];
}));

// Basic with multiple steps of parallel commands

test("testFn should pass (basicMultistepParallel)", testFn(basicMultistepParallel, function () {
  var c1 = [cmds.echo("foo"), cmds.echo("foo")];
  var c2 = [cmds.echo("foo"), cmds.echo("foo")];
  // prettier-ignore
  return [[['foo'], c1], [['foo', 'foo'], c2], [['foo', 'foo'], { s1: 'foo1', s2: 'foo2', s3: 'foo3', s4: 'foo4' }]];
}));

test("testFn semantic should pass (basicMultistepParallel)", testFn(basicMultistepParallel, function () {
  var c1 = [cmds.echo("foo"), cmds.echo("foo")];
  var c2 = [cmds.echo("foo"), cmds.echo("foo")];
  // prettier-ignore
  return args('foo').cmd(c1).result(['foo', 'foo']).cmd(c2).result(['foo', 'foo']).returns({ s1: 'foo1', s2: 'foo2', s3: 'foo3', s4: 'foo4' });
}));

test("testFnV2 should pass (basicMultistepParallel)", testFnV2(basicMultistepParallel, function () {
  var c1 = [cmds.echo("foo"), cmds.echo("foo")];
  var c2 = [cmds.echo("foo"), cmds.echo("foo")];
  // prettier-ignore
  return [['foo'], [c1, ['foo', 'foo']], [c2, ['foo', 'foo']], { s1: 'foo1', s2: 'foo2', s3: 'foo3', s4: 'foo4' }];
}));

// Basic with empty args

test("testFn should pass (basicEmpty)", testFn(basicEmpty, function () {
  // prettier-ignore
  return [[[null], []], [[], []]];
}));

test("testFn semantic should pass (basicEmpty)", testFn(basicEmpty, function () {
  // prettier-ignore
  return args(null).cmd([]).result([]).returns([]);
}));

test("testFnV2 should pass (basicEmpty)", testFnV2(basicEmpty, function () {
  // prettier-ignore
  return [[null], [[], []], []];
}));

// Interpreter error handling

test("testFn should handle errors (usesThrowingInterpreter)", testFn(usesThrowingInterpreter, function () {
  // prettier-ignore
  return [[[null], cmds.die('oops')], [new Error('oops!'), new Error('oops!')]];
}));

test("testFn semantic should handle errors (usesThrowingInterpreter)", testFn(usesThrowingInterpreter, function () {
  // prettier-ignore
  return args([null]).cmd(cmds.die('oops')).result(new Error('oops!')).returns(new Error('oops!'));
}));

test("testFn semantic should handle errors using interpreterThrows and throws (usesThrowingInterpreter)", testFn(usesThrowingInterpreter, function () {
  // prettier-ignore
  return args([null]).cmd(cmds.die('oops')).interpreterThrows(new Error('oops!')).throws(new Error('oops!'));
}));

function throwImmediately() {
  return regeneratorRuntime.wrap(function throwImmediately$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          throw new Error("oops!");

        case 1:
        case "end":
          return _context6.stop();
      }
    }
  }, _marked5, this);
}

test("testFn semantic should handle errors thrown immediately", testFn(throwImmediately, function () {
  // prettier-ignore
  return args().throws(new Error('oops!'));
}));

test("testFn semantic should handle errors without returns (usesThrowingInterpreter)", testFn(usesThrowingInterpreter, function () {
  // prettier-ignore
  return args([null]).cmd(cmds.die('oops')).throws(new Error('oops!'));
}));

test("testFnV2 should handle errors (usesThrowingInterpreter)", testFnV2(usesThrowingInterpreter, function () {
  // prettier-ignore
  return [[null], [cmds.die('oops'), new Error('oops!')], new Error('oops!')];
}));

// Return cmd result
// Throw from semantic test builder

function returnCmdResult() {
  return regeneratorRuntime.wrap(function returnCmdResult$(_context7) {
    while (1) {
      switch (_context7.prev = _context7.next) {
        case 0:
          _context7.next = 2;
          return cmds.echo("foo");

        case 2:
          return _context7.abrupt("return", _context7.sent);

        case 3:
        case "end":
          return _context7.stop();
      }
    }
  }, _marked6, this);
}

test("testFn semantic should return cmd result", testFn(returnCmdResult, function () {
  // prettier-ignore
  return args().cmd(cmds.echo('foo')).returns("foo");
}));

// Throw from semantic test builder

function throwSemantic() {
  var value;
  return regeneratorRuntime.wrap(function throwSemantic$(_context8) {
    while (1) {
      switch (_context8.prev = _context8.next) {
        case 0:
          _context8.next = 2;
          return cmds.echo("foo");

        case 2:
          value = _context8.sent;
          throw new Error("oops");

        case 4:
        case "end":
          return _context8.stop();
      }
    }
  }, _marked7, this);
}

test("testFn semantic should throw if .throws is used", testFn(throwSemantic, function () {
  // prettier-ignore
  return args().cmd(cmds.echo('foo')).result('foo').throws(new Error("oops"));
}));

test("testFn should throw proper error if function throws incorrect error", function () {
  try {
    testFn(throwSemantic, function () {
      // prettier-ignore
      return args().cmd(cmds.echo('foo')).result('foo').throws(new Error("wrong"));
    })();
  } catch (e) {
    deepEqual(e.stack.includes("Error on Step 2"), true);
    return;
  }
  throw new Error("Failed: Did not throw");
});

// Single line

test("single line should pass", testSingleLine(function () {
  // prettier-ignore
  return [[['123'], cmds.httpGet('http://example.com/api/v1/users/123')], [{ foo: 'bar' }, { foo: 'bar' }]];
}));

test("testFn semantic single line should not fail", testSingleLine(function () {
  // prettier-ignore
  return args('123').cmd(cmds.httpGet('http://example.com/api/v1/users/123')).result({ foo: 'bar' }).returns({ foo: 'bar' });
}));

test("testFn should give proper error message if yielding array but no results", function () {
  try {
    testYieldArray(function () {
      // prettier-ignore
      return [[[], [{ type: 'test' }]]];
    })();
  } catch (e) {
    deepEqual(e.message, "Your spec does not have as many steps as your function.  Are you missing a return line?");
  }
});

test("testFn should give proper error message if spec is returning undefined", function () {
  try {
    testYieldArray(function () {})();
  } catch (e) {
    deepEqual(e.message, 'Your spec must return an array of tuples.  It is currently returning a value of type "undefined".');
  }
});

test("testFn should give proper error message if spec is returning an object", function () {
  try {
    testYieldArray(function () {
      return {};
    })();
  } catch (e) {
    deepEqual(e.message, 'Your spec must return an array of tuples.  It is currently returning a value of type "object".');
  }
});

test("testFn should give proper error message if spec is returning an string", function () {
  try {
    testYieldArray(function () {
      return "what?";
    })();
  } catch (e) {
    deepEqual(e.message, 'Your spec must return an array of tuples.  It is currently returning a value of type "string".');
  }
});
//# sourceMappingURL=test-framework.spec.js.map