"use strict";

var _require = require("./test-util"),
    errorToJson = _require.errorToJson,
    expectErrorEquality = _require.expectErrorEquality;

test("errorToJson()", function () {
  var error = new Error("oops");
  error.code = "OOPS";

  var jsonError = errorToJson(error);

  expect(jsonError).toEqual({
    name: "Error",
    message: "oops",
    code: "OOPS",
    stack: error.stack
  });
});

test("expectErrorEquality() should not throw if errors are equal", function () {
  var error1 = new Error("foo");
  expectErrorEquality(error1, error1);
});

test("expectErrorEquality() should throw if errors are different", function () {
  var error1 = new Error("foo");
  var error2 = new Error("bar");
  expect(function () {
    return expectErrorEquality(error1, error2);
  }).toThrow();
});

test("expectErrorEquality() should convert string to errors (right)", function () {
  var error1 = new Error("foo");
  expectErrorEquality(error1, "foo");
});

test("expectErrorEquality() should convert string to errors (left)", function () {
  var error1 = new Error("foo");
  expectErrorEquality("foo", error1);
});

test("expectErrorEquality() should throw if errors are using strings", function () {
  var error1 = new Error("foo");
  expect(function () {
    return expectErrorEquality(error1, "bar");
  }).toThrow();
});

test("expectErrorEquality() should not throw even if stack traces are different", function () {
  var error1 = new Error("foo");
  var error2 = new Error("foo");
  expectErrorEquality(error1, error2);
});

test("expectErrorEquality() should not throw if an error is compared to null (right)", function () {
  var error1 = new Error("foo");
  expect(function () {
    return expectErrorEquality(error1, null);
  }).toThrow();
});

test("expectErrorEquality() should not throw if an error is compared to null (left)", function () {
  var error1 = new Error("foo");
  expect(function () {
    return expectErrorEquality(null, error1);
  }).toThrow();
});
//# sourceMappingURL=test-util.spec.js.map